/**
 * @mainpage GP3LIB
 * This is the prototypical C library for the GP3 I/O board from
 * AWC (see <A HREF=http://www.awce.com/gp3.htm>http://www.awce.com/gp3.htm</A>)
 * 
 * On Windows you may prefer to use the ActiveX controls provided for the 
 * kit in both control form (for VB, Delphi, VC++, etc. or as an ActiveX
 * DLL (useful for WSH, ASP, etc.). In addition, there are older binary-only
 * standard DLLs that are not generated from this source.
 *
 * The strength of this library is that it has two parts: a portable part,
 * and a platform-specific part. AWC provides platform-specific parts
 * for Linux, Win32, and Cygwin. It should be relatively easy to
 * port the library to other C platforms by simply rewriting the platform-
 * specific part.
 *
 * @author Al Williams alw@al-williams.com
 * @version 1.1
 * 
 * Documentation version 1
 * For access to this documentation, access http://www.awce.com/gp3doc.zip
 */
 
/** @file
 * This is the header included by the GP3 library and all programs that use it.
 * @author Al Williams alw@al-williams.com
 * @version 1.1
 * @date 17 Feb 2004
 */



#ifndef _AWC_GP3LIB_H
#define _AWC_GP3LIB_H

/** @defgroup PlatformSpecific Platform Specific Routines
 * Platform-specific functions
 * You must implement these to port the library to a new platform.
 * @{
 */

/** Open the GP3 serial port.
 *   @param port The name of the port (varies by platform, e.g., COM1, /dev/cua0, etc.)
 *   @return -1 for error
 */
int gp3openport(char *port);  // returns -1 for error
/** Close the GP3 port.
 */
void gp3closeport();
/** Write a byte to the GP3.
 * This function blocks if the GP3 is not ready.
 * @param byte The byte to write
 * @return 1 for success
 */
int gp3write(int byte);   // returns 1 for success
/** Read a byte from the GP3.
 * This function blocks until a byte is received
 * @return Character or -1 in case of error
 */
unsigned int gp3read(void);  // returns character or -1 for error
/** Optional function reads the CTS input to determine if the GP3 is ready.
 * None of the functions use this (although the demo program does).
 * Using this function it is possible to avoid calling gp3write when
 * it would cause the program to block.
 * @return 1 if GP3 is ready; 0 otherwise
 */
int gp3ready(void);  // optional returns 0 for not ready, 1 for ready

/** @} */

/** @defgroup Common Common Portable Routines
 *  Common (portable) functions 
 * You should not need to change these functions.
 * @{
*/

/** Read a 16-bit word from the GP3
   @return The 16-bit word read from the GP3
*/
unsigned int gp3readword(void);

/** Set the GP3's onboard LED.
    @param onoff 0 for LED off; 1 for LED on
*/
void gp3setLED(int onoff);
/** Read an analog channel. 
    @param chan Channel number (0-4)
    @return A value between 0 (0V) and 1023 (5V)
*/
unsigned int gp3a2d(int chan);

/** Set a GP3 pin High
 * The pin is automatically made an output.
 * @param pin The pin number (0-7)
 */
void gp3high(int pin);
/** Set a GP3 pin Low
 * The pin is automatically made an output.
 * @param pin The pin number (0-7)
 */
void gp3low(int pin);
/** Toggle a GP3 output pin (that is set from low to high or high to low)
 * The pin is automatically made an output.
 * @param pin The pin number (0-7)
 */
void gp3toggle(int pin);

/** Read a GP3 input pin.
 * The pin is automatically made an input.
 * @param pin The pin number (0-7)
 * @return 0 or 1 (depending on the state of the pin)
 */
int gp3input(int pin);
/** Set the GP3 direction register.
 * The tris parameter has 8 bits that each correspond to a GP3 digital I/O pin 
 * A 0 in a tris bit indicates an output bit and a 1 indicates an input bit
 * This overrides previous calls to gp3high, gp3low, gp3input, etc.
 * @param tris The direction bits
 */
void gp3settris(int tris);
/** Get the GP3 direction register.
 * The tris parameter has 8 bits that each correspond to a GP3 digital I/O pin 
 * A 0 in a tris bit indicates an output bit and a 1 indicates an input bit
 * @return The direction bits
 */
unsigned int gp3gettris(void);
/** Set all output pins at once (does not affect input pins).
 * @param byte The output byte
 */
void gp3setpins(int byte);
/** Read all input pins at once.
 * @return Input state.
 */
unsigned int gp3getpins(void);
/** Write to EEPROM.
 * @param address EEPROM address (0-127)
 * @param byte Byte to write to EEPROM
 */
void gp3writeee(int address, int byte);
/** Read from EEPROM.
 * @param address EEPROM address (0-127)
 * @return Byte from EEPROM
 */
unsigned int gp3readee(int address);
/** Generate PWM.
 * @param chan The digital pin to use or -1 for hardware PWM.If a digital pin, the GP3 automatically makes the pin an output.
 * @param dc The duty cycle (0-255, where 256=100%)
 * @param dfreq For hardware PWM, the frequency in Hz; otherwise the duration of the PWM pulses in approximate milliseconds (frequency between 1225 and 32767; duration between 0 and 65535) 
 */
void gp3pwm(int chan,int dc, int dfreq);
/** Count transitions on a digital input. The pin automatically becomes an input.
 * This command always counts rising edges. Sampling occurs every 4uS.
 * @param pin The pin to use
 * @param ms The time to sample in milliseconds
 */
unsigned int gp3count(int pin,unsigned int ms);
/** Read an RC charge/discharge time.
 * This command allows you to read the amount of time required
 * for an RC (resistor/capacitor) network to charge or discharge.
 * It is useful for measuring resistive or capacitive sensors like
 * thermistors, pots, etc.
 * @param pin The pin to use (forced to input)
 * @param state The desired state (0 or 1)
 * @return The time required in 2uS units.
 */
unsigned int gp3rctime(int pin, int state);
/** Measure a single pulse.
 * @param pin The pin to use (forced to input)
 * @param state 0 to read a low pulse or 1 to read a high pulse
 * @return The pulse width measured in 2uS units
 */ 
unsigned int gp3pulsein(int pin, int state);
/** Generate a single pulse.
 * This command inverts the output pin's state for the specified duration.
 * @param pin The pin to use (forced to output)
 * @param duration The pulse width in 2uS units
 */
void gp3pulseout(int pin, int duration);
/** Generate a psuedo sine wave.
 * @param pin The pin to use (forced to output)
 * @param ms The duration of the sine wave in milliseconds
 * @param freq The frequency in Hz (32767 maximum)
 */
void gp3freq(int pin, int ms, int freq);
/** Set hardware counter options.
 * @param prescale Prescale factor (1, 2, 4, or 8)
 * @param ext Set to 1 for external counter, or 0 for internal 5MHz clock
 */
void gp3setcounter(int prescale, int ext);
/** Read hardware counter and reset to 0.
 * @return 16-bit counter value.
 */
unsigned int gp3counter(void);
/** Reset GP3 to power up state. */
void gp3resetall(void);
/** Reset GP3's communicatios state. */
void gp3reset(void);
/** Repeat next command. Eligible commands include 
 * gp3a2d(), gp3input(), gp3rctime(),gp3getpins(), gp3gettris(), and gp3pulsein()
 * If you set the count to 100, for example, and then issued one gp3a2d()
 * command, you'd follow it with 99 gp3readword() calls to fetch the other 
 * 99 A/D samples.
 * @note NOTE: For commands that return bytes, call gp3read();
 * for commands that return words, call gp3readword()
 * @param count The number of times to repeat the next command
 */
void gp3repeat(int count);
/** Increment the internal argument. You should not need to call
 * this function under normal circumstances.
 */
void gp3argincr(void);
/** Decrement the internal argument. You should not need to call
 * this function under normal circumstances.
 */
void gp3argdecr(void);
/** Check the GP3 status. If the GP3 is ready and connected it 
 * will return a version number (currently 0x33).
 * @return 0x33
 */
unsigned int gp3check(void);
/** @} */
#endif


